@@ -421,6 +421,7 @@ def generateFunctionBodyCode( function_body, context ):
             parameters             = parameters,
             closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
+            temp_variables         = function_body.getTempVariables(),
             source_ref             = function_body.getSourceReference(),
             function_codes         = function_codes,
             function_doc           = function_body.getDoc()
@@ -433,6 +434,7 @@ def generateFunctionBodyCode( function_body, context ):
             parameters             = parameters,
             closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
+            temp_variables         = function_body.getTempVariables(),
             function_codes         = function_codes,
             function_doc           = function_body.getDoc(),
             file_scope             = Generator.getExportScopeCode(
@@ -726,7 +728,7 @@ def generateCallCode( call_node, context ):
                    ( None, ) * len( call_args.getConstant() ),
                 called_identifier = called_identifier,
                 arguments         = [
-                    Generator.getConstantHandle(
+                    Generator.getConstantAccess(
                         constant = element,
                         context  = context
                     )
@@ -840,7 +842,7 @@ def getOrderRelevance( expressions, allow_none = False ):
 
     return result
 
-def generateExpressionCode( expression, context, allow_none = False ):
+def _generateExpressionCode( expression, context, allow_none ):
     # This is a dispatching function with a branch per expression node type, and
     # therefore many statements even if every branch is small
     # pylint: disable=R0912,R0915
@@ -848,6 +850,8 @@ def generateExpressionCode( expression, context, allow_none = False ):
     if expression is None and allow_none:
         return None
 
+    # Make sure we don't generate code twice for any node, this uncovers bugs
+    # where nodes are shared in the tree, which is not allowed.
     assert not hasattr( expression, "code_generated" ), expression
     expression.code_generated = True
 
@@ -1260,6 +1264,10 @@ def generateExpressionCode( expression, context, allow_none = False ):
                 context  = context
             )
         )
+    elif expression.isExpressionBuiltinSet():
+        identifier = Generator.getBuiltinSetCode(
+            identifier = makeExpressionCode( expression.getValue() )
+        )
     elif Utils.python_version < 300 and expression.isExpressionBuiltinStr():
         identifier = Generator.getBuiltinStrCode(
             identifier = makeExpressionCode( expression.getValue() )
@@ -1356,7 +1364,9 @@ def generateExpressionCode( expression, context, allow_none = False ):
     elif expression.isExpressionAssignmentTempKeeper():
         identifier = Generator.getAssignmentTempKeeperCode(
             variable          = expression.getVariable(),
-            source_identifier = makeExpressionCode( expression.getAssignSource() ),
+            source_identifier = makeExpressionCode(
+                expression.getAssignSource()
+            ),
             context           = context
         )
     elif expression.isExpressionBuiltinInt():
@@ -1513,6 +1523,22 @@ def generateExpressionCode( expression, context, allow_none = False ):
 
     return identifier
 
+def generateExpressionCode( expression, context, allow_none = False ):
+    try:
+        return _generateExpressionCode(
+            expression = expression,
+            context    = context,
+            allow_none = allow_none
+        )
+    except:
+        Tracing.printError(
+            "Problem with %r at %s" % (
+                expression,
+                expression.getSourceReference()
+            )
+        )
+        raise
+
 def generateAssignmentVariableCode( variable_ref, value, context ):
     return Generator.getVariableAssignmentCode(
         variable   = variable_ref.getVariable(),
@@ -2130,16 +2156,6 @@ def generateLoopCode( statement, context ):
         needs_continue_exception = statement.needsExceptionContinue()
     )
 
-def generateTempBlock( statement, context ):
-    body_codes = generateStatementSequenceCode(
-        statement_sequence = statement.getBody(),
-        context            = context
-    )
-
-    return Generator.getBlockCode(
-        body_codes
-    )
-
 def generateReturnCode( statement, context ):
     return Generator.getReturnCode(
         identifier    = generateExpressionCode(
@@ -2165,83 +2181,22 @@ def generateGeneratorReturnCode( statement, context ):
 def generateStatementCode( statement, context ):
     try:
         statement_context = Contexts.PythonStatementContext( context )
-
-        # Special hack for deferred declaration of temporary variables at the
-        # first opportunity.
-        if statement.isStatementAssignmentVariable():
-            variable_ref = statement.getTargetVariableRef()
-            variable = variable_ref.getVariable()
-
-            if variable.isTempVariableReference():
-                referenced = variable.getReferenced()
-
-                if not referenced.isDeclared():
-                    referenced.markAsDeclared()
-
-                    assign_source = generateExpressionCode(
-                        expression = statement.getAssignSource(),
-                        context    = statement_context
-                    )
-
-                    local_inits = Generator.getTempKeeperDecl(
-                        context = statement_context
-                    )
-
-                    if not local_inits:
-                        return Generator.getLocalVariableInitCode(
-                            context   = statement_context,
-                            variable  = variable.getReferenced(),
-                            init_from = assign_source
-                        )
-                    else:
-                        if referenced.getNeedsFree():
-                            return """\
-PyObject *_tmp_%s;
-{
-%s
-    _tmp_%s = %s;
-}
-%s _%s( _tmp_%s );""" % (
-                            referenced.getCodeName(),
-                            Generator.indented( local_inits ),
-                            referenced.getCodeName(),
-                            assign_source.getCodeExportRef(),
-                            referenced.getDeclarationTypeCode( False ),
-                            referenced.getCodeName(),
-                            referenced.getCodeName(),
-
-                        )
-                        else:
-                            return """\
-%s _%s;
-{
-%s
-    _%s = %s;
-}
-""" % (
-                            referenced.getDeclarationTypeCode( False ),
-                            referenced.getCodeName(),
-                            Generator.indented( local_inits ),
-                            referenced.getCodeName(),
-                            assign_source.getCodeExportRef(),
-
-                        )
-
-
-
-
         result = _generateStatementCode( statement, statement_context )
         local_inits = Generator.getTempKeeperDecl( statement_context )
 
-
         if local_inits:
             result = Generator.getBlockCode(
-                local_inits + [ result ]
+                local_inits + result.split( "\n" )
             )
 
         return result
     except:
-        Tracing.printError( "Problem with %r at %s" % ( statement, statement.getSourceReference() ) )
+        Tracing.printError(
+            "Problem with %r at %s" % (
+                statement,
+                statement.getSourceReference()
+            )
+        )
         raise
 
 def _generateStatementCode( statement, context ):
@@ -2315,11 +2270,6 @@ def _generateStatementCode( statement, context ):
             statement = statement,
             context   = context
         )
-    elif statement.isStatementTempBlock():
-        code = generateTempBlock(
-            statement = statement,
-            context   = context
-        )
     elif statement.isStatementExpressionOnly():
         code = Generator.getStatementCode(
             identifier = makeExpressionCode(
@@ -2632,6 +2582,7 @@ def generateModuleCode( global_context, module, module_name, other_modules ):
         ],
         function_decl_codes = function_decl_codes,
         function_body_codes = function_body_codes,
+        temp_variables      = module.getTempVariables(),
         context             = context,
     )
 
