@@ -20,17 +20,18 @@
 Right now only the creation is done here. But more should be added later on.
 """
 
-from .Identifiers import CallIdentifier
+from .Identifiers import HelperCallIdentifier
 
 from .TupleCodes import getTupleCreationCode
 
-def getSetCreationCode( context, element_identifiers ):
+def getSetCreationCode( context, order_relevance, element_identifiers ):
     tuple_identifier = getTupleCreationCode(
+        order_relevance     = order_relevance,
         element_identifiers = element_identifiers,
         context             = context
     )
 
-    return CallIdentifier(
-        called  = "MAKE_SET",
-        args    = ( tuple_identifier.getCodeTemporaryRef(), )
+    return HelperCallIdentifier(
+        "MAKE_SET",
+        tuple_identifier
     )
