@@ -26,8 +26,9 @@ from nuitka.odict import OrderedDict
 from nuitka.oset import OrderedSet
 
 from nuitka import (
+    Variables,
     Tracing,
-    TreeXML
+    TreeXML,
 )
 
 from nuitka.__past__ import iterItems
@@ -77,10 +78,11 @@ class NodeCheckMetaClass( type ):
 
         type.__init__( mcs, name, bases, dictionary )
 
-# For every node type, there is a test, and then some more members, pylint: disable=R0904
+# For every node type, there is a test, and then some more members,
+# pylint: disable=R0904
 
-# For Python2/3 compatible source, we create a base class that has the metaclass used and
-# doesn't require making a choice.
+# For Python2/3 compatible source, we create a base class that has the metaclass
+# used and doesn't require making a choice.
 NodeMetaClassBase = NodeCheckMetaClass( "NodeMetaClassBase", (object, ), {} )
 
 class NodeBase( NodeMetaClassBase ):
@@ -102,7 +104,8 @@ class NodeBase( NodeMetaClassBase ):
         return True
 
     def __repr__( self ):
-        # This is to avoid crashes, because of bugs in detail. pylint: disable=W0702
+        # This is to avoid crashes, because of bugs in detail.
+        # pylint: disable=W0702
         try:
             detail = self.getDetail()
         except:
@@ -114,7 +117,8 @@ class NodeBase( NodeMetaClassBase ):
             return "<Node %s %s>" % ( self.getDescription(), detail )
 
     def getDescription( self ):
-        """ Description of the node, intented for use in __repr__ and graphical display.
+        """ Description of the node, intented for use in __repr__ and
+            graphical display.
 
         """
         return "%s at %s" % ( self.kind, self.source_ref.getAsString() )
@@ -127,7 +131,8 @@ class NodeBase( NodeMetaClassBase ):
         return {}
 
     def getDetail( self ):
-        """ Details of the node, intended for use in __repr__ and graphical display.
+        """ Details of the node, intended for use in __repr__ and graphical
+            display.
 
         """
         # Virtual method, pylint: disable=R0201
@@ -143,6 +148,26 @@ class NodeBase( NodeMetaClassBase ):
 
         return self.parent
 
+    def getParents( self ):
+        """ Parents of the node. Up to module level.
+
+        """
+        result = []
+        current = self
+
+        while True:
+            current = current.getParent()
+
+            result.append( current )
+
+            if current.isPythonModule() or current.isExpressionFunctionBody():
+                break
+
+        assert None not in result, self
+
+        result.reverse()
+        return result
+
     def getParentFunction( self ):
         """ Return the parent that is a function.
 
@@ -610,6 +635,10 @@ class ClosureGiverNodeBase( CodeNodeBase ):
 
         self.keeper_variables = OrderedSet()
 
+        self.temp_variables = OrderedDict()
+
+        self.temp_scopes = OrderedDict()
+
     def hasProvidedVariable( self, variable_name ):
         return variable_name in self.providing
 
@@ -639,11 +668,9 @@ class ClosureGiverNodeBase( CodeNodeBase ):
     def getProvidedVariables( self ):
         return self.providing.values()
 
-    def getTempKeeperVariable( self ):
+    def allocateTempKeeperVariable( self ):
         name = "keeper_%d" % len( self.keeper_variables )
 
-        from nuitka import Variables
-
         result = Variables.TempKeeperVariable(
             owner         = self,
             variable_name = name
@@ -653,6 +680,51 @@ class ClosureGiverNodeBase( CodeNodeBase ):
 
         return result
 
+    def getTempKeeperVariables( self ):
+        return self.keeper_variables
+
+    def removeTempKeeperVariable( self, variable ):
+        self.keeper_variables.discard( variable )
+
+    def allocateTempScope( self, name, allow_closure = False ):
+        self.temp_scopes[ name ] = self.temp_scopes.get( name, 0 ) + 1
+
+        # TODO: Instead of using overly long code name, could just visit parents
+        # and make sure to allocate the scope at the top.
+        if allow_closure:
+            return "%s_%s_%d" % (
+                self.getCodeName(),
+                name,
+                self.temp_scopes[ name ]
+            )
+        else:
+            return "%s_%d" % ( name, self.temp_scopes[ name ] )
+
+    def allocateTempVariable( self, temp_scope, name ):
+        if temp_scope is not None:
+            full_name = "%s__%s" % ( temp_scope, name )
+        else:
+            full_name = name
+
+        del name
+
+        assert full_name not in self.temp_variables, full_name
+
+        result = Variables.TempVariable(
+            owner         = self,
+            variable_name = full_name
+        )
+
+        self.temp_variables[ full_name ] = result
+
+        return result
+
+    def getTempVariables( self ):
+        return tuple( self.temp_variables.values() )
+
+    def removeTempVariable( self, variable ):
+        del self.temp_variables[ variable.getName() ]
+
 
 class ParameterHavingNodeBase( ClosureGiverNodeBase ):
     def __init__( self, name, code_prefix, parameters, source_ref ):
@@ -750,21 +822,18 @@ class ClosureTakerMixin:
 
 
 class ExpressionMixin:
-    def getValueFriend( self, constraint_collection ):
-        return self
-
     def isCompileTimeConstant( self ):
         """ Has a value that we can use at compile time.
 
-            Yes or no. If it has such a value, simulations can be applied at compile time
-            and e.g. operations or conditions, or even calls may be executed against it.
+            Yes or no. If it has such a value, simulations can be applied at
+            compile time and e.g. operations or conditions, or even calls may
+            be executed against it.
         """
         # Virtual method, pylint: disable=R0201
-
         return False
 
     def getCompileTimeConstant( self ):
-        assert self.isCompileTimeConstant(), ( self, "asked for compile time constant" )
+        assert self.isCompileTimeConstant(), self
 
         assert False
 
@@ -794,10 +863,6 @@ class ExpressionMixin:
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def getIterationNext( self ):
-        # Virtual method, pylint: disable=R0201,W0613
-        return None
-
     def mayProvideReference( self ):
         """ May at run time produce a reference.
 
@@ -810,7 +875,7 @@ class ExpressionMixin:
     def getIterationLength( self ):
         """ Value that "len" or "PyObject_Size" would give, if known.
 
-        Otherwise it is "None" to indicate unknown.
+            Otherwise it is "None" to indicate unknown.
         """
 
         # Virtual method, pylint: disable=R0201
@@ -824,15 +889,15 @@ class ExpressionMixin:
     def getStrValue( self ):
         """ Value that "str" or "PyObject_Str" would give, if known.
 
-        Otherwise it is "None" to indicate unknown.
+            Otherwise it is "None" to indicate unknown.
         """
         string_value = self.getStringValue()
 
         if string_value is not None:
             from .NodeMakingHelpers import makeConstantReplacementNode
 
-            # TODO: Side effects should be considered, getStringValue may be omitting
-            # effects.
+            # TODO: Side effects should be considered, getStringValue may be
+            # omitting effects.
 
             return makeConstantReplacementNode(
                 node     = self,
@@ -895,6 +960,7 @@ class ExpressionMixin:
     def onContentEscapes( self, constraint_collection ):
         pass
 
+
 class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def __init__( self ):
         self.computed_attribute = False
@@ -902,8 +968,9 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def isCompileTimeConstant( self ):
         """ Has a value that we can use at compile time.
 
-            Yes or no. If it has such a value, simulations can be applied at compile time
-            and e.g. operations or conditions, or even calls may be executed against it.
+            Yes or no. If it has such a value, simulations can be applied at
+            compile time and e.g. operations or conditions, or even calls may
+            be executed against it.
         """
         # Virtual method, pylint: disable=R0201
 
@@ -921,7 +988,8 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
         return getComputationResult(
             node        = not_node,
             computation = lambda : not self.getCompileTimeConstant(),
-            description = "Compile time constant negation truth value precomputed."
+            description = """\
+Compile time constant negation truth value precomputed."""
         )
 
 
@@ -938,7 +1006,9 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
             return getComputationResult(
                 node        = lookup_node,
                 computation = lambda : getattr( value, attribute_name ),
-                description = "Attribute lookup to %s precomputed." % attribute_name
+                description = "Attribute lookup to %s precomputed." % (
+                    attribute_name
+                )
             )
 
         self.computed_attribute = True
@@ -961,7 +1031,8 @@ class CompileTimeConstantExpressionMixin( ExpressionMixin ):
     def computeExpressionSlice( self, lookup_node, lower, upper, constraint_collection ):
         from .NodeMakingHelpers import getComputationResult
 
-        # TODO: Could be happy with predictable index values and not require constants.
+        # TODO: Could be happy with predictable index values and not require
+        # constants.
         if lower is not None:
             if upper is not None:
                 if lower.isCompileTimeConstant() and upper.isCompileTimeConstant():
@@ -1054,8 +1125,8 @@ class ExpressionBuiltinNoArgBase( NodeBase, ExpressionMixin ):
     def computeExpression( self, constraint_collection ):
         from .NodeMakingHelpers import getComputationResult
 
-        # The lamba is there for make sure that no argument parsing will reach the builtin
-        # function at all, pylint: disable=W0108
+        # The lamba is there for make sure that no argument parsing will reach
+        # the builtin function at all, pylint: disable=W0108
         return getComputationResult(
             node        = self,
             computation = lambda : self.builtin_function(),
