@@ -176,6 +176,12 @@ def buildVariableReferenceNode( provider, node, source_ref ):
         source_ref    = source_ref
     )
 
+# Python3.4 only, True and False, are not given as variables anymore.
+def buildNamedConstantNode( node, source_ref ):
+    return ExpressionConstantRef(
+        constant   = node.value,
+        source_ref = source_ref
+    )
 
 def buildSequenceCreationNode( provider, node, source_ref ):
     return makeSequenceCreationOrConstant(
@@ -187,9 +193,10 @@ def buildSequenceCreationNode( provider, node, source_ref ):
 
 def buildDictionaryNode( provider, node, source_ref ):
     return makeDictCreationOrConstant(
-        keys          = buildNodeList( provider, node.keys, source_ref ),
-        values        = buildNodeList( provider, node.values, source_ref ),
-        source_ref    = source_ref
+        keys       = buildNodeList( provider, node.keys, source_ref ),
+        values     = buildNodeList( provider, node.values, source_ref ),
+        lazy_order = False,
+        source_ref = source_ref
     )
 
 def buildConditionNode( provider, node, source_ref ):
@@ -791,6 +798,7 @@ setBuildDispatchers(
         "IfExp"        : buildConditionalExpressionNode,
     },
     path_args2 = {
+        "NameConstant" : buildNamedConstantNode,
         "Import"       : buildImportModulesNode,
         "Str"          : buildStringNode,
         "Num"          : buildNumberNode,
